@@ -1,6 +1,5 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from collections import defaultdict
 import time
 import logging
 import os
@@ -11,12 +10,11 @@ from decimal import Decimal
 from threading import RLock
 
 try:
-    from psycopg2 import connect
-except ImportError:
     from psycopg2cffi import compat
     compat.register()
-    from psycopg2 import connect
-from psycopg2 import Binary
+except ImportError:
+    pass
+from psycopg2 import connect, Binary
 from psycopg2.pool import ThreadedConnectionPool, PoolError
 from psycopg2.extensions import cursor
 from psycopg2.extensions import ISOLATION_LEVEL_REPEATABLE_READ
@@ -31,12 +29,14 @@ from psycopg2 import IntegrityError as DatabaseIntegrityError
 from psycopg2 import OperationalError as DatabaseOperationalError
 from psycopg2.extras import register_default_json, register_default_jsonb
 
-from sql import Flavor
+from sql import Flavor, Cast
 from sql.functions import Function
+from sql.operators import BinaryOperator
 
 from trytond.backend.database import DatabaseInterface, SQLType
 from trytond.config import config, parse_uri
 from trytond.protocols.jsonrpc import JSONDecoder
+from trytond.tools.gevent import is_gevent_monkey_patched
 
 __all__ = ['Database', 'DatabaseIntegrityError', 'DatabaseOperationalError']
 
@@ -80,10 +80,35 @@ class TryAdvisoryLock(Function):
     _function = 'pg_try_advisory_xact_lock'
 
 
+class JSONBExtractPath(Function):
+    __slots__ = ()
+    _function = 'jsonb_extract_path'
+
+
+class JSONKeyExists(BinaryOperator):
+    __slots__ = ()
+    _operator = '?'
+
+
+class JSONAnyKeyExist(BinaryOperator):
+    __slots__ = ()
+    _operator = '?|'
+
+
+class JSONAllKeyExist(BinaryOperator):
+    __slots__ = ()
+    _operator = '?&'
+
+
+class JSONContains(BinaryOperator):
+    __slots__ = ()
+    _operator = '@>'
+
+
 class Database(DatabaseInterface):
 
     _lock = RLock()
-    _databases = defaultdict(dict)
+    _databases = {}
     _connpool = None
     _list_cache = {}
     _list_cache_timestamp = {}
@@ -105,14 +130,13 @@ class Database(DatabaseInterface):
     def __new__(cls, name='template1'):
         with cls._lock:
             now = datetime.now()
-            databases = cls._databases[os.getpid()]
-            for database in list(databases.values()):
+            for database in list(cls._databases.values()):
                 if ((now - database._last_use).total_seconds() > _timeout
                         and database.name != name
                         and not database._connpool._used):
                     database.close()
-            if name in databases:
-                inst = databases[name]
+            if name in cls._databases:
+                inst = cls._databases[name]
             else:
                 if name == 'template1':
                     minconn = 0
@@ -121,10 +145,9 @@ class Database(DatabaseInterface):
                 inst = DatabaseInterface.__new__(cls, name=name)
                 logger.info('connect to "%s"', name)
                 inst._connpool = ThreadedConnectionPool(
-                    minconn, _maxconn,
-                    cursor_factory=LoggingCursor,
-                    **cls._connection_params(name))
-                databases[name] = inst
+                    minconn, _maxconn, **cls._connection_params(name),
+                    cursor_factory=LoggingCursor)
+                cls._databases[name] = inst
             inst._last_use = datetime.now()
             return inst
 
@@ -161,17 +184,11 @@ class Database(DatabaseInterface):
                     time.sleep(1)
                     continue
                 raise
-        # We do not use set_session because psycopg2 < 2.7 and psycopg2cffi
-        # change the default_transaction_* attributes which breaks external
-        # pooling at the transaction level.
         if autocommit:
             conn.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
         else:
             conn.set_isolation_level(ISOLATION_LEVEL_REPEATABLE_READ)
-        # psycopg2cffi does not have the readonly property
-        if hasattr(conn, 'readonly'):
-            conn.readonly = readonly
-        elif not autocommit and readonly:
+        if readonly:
             cursor = conn.cursor()
             cursor.execute('SET TRANSACTION READ ONLY')
         return conn
@@ -183,7 +200,7 @@ class Database(DatabaseInterface):
         with self._lock:
             logger.info('disconnect from "%s"', self.name)
             self._connpool.closeall()
-            self._databases[os.getpid()].pop(self.name)
+            self._databases.pop(self.name)
 
     @classmethod
     def create(cls, connection, database_name, template='template0'):
@@ -220,13 +237,10 @@ class Database(DatabaseInterface):
             res = []
             for db_name, in cursor:
                 try:
-                    conn = connect(**self._connection_params(db_name))
-                    try:
-                        with conn:
-                            if self._test(conn, hostname=hostname):
-                                res.append(db_name)
-                    finally:
-                        conn.close()
+                    with connect(**self._connection_params(db_name)
+                            ) as conn:
+                        if self._test(conn, hostname=hostname):
+                            res.append(db_name)
                 except Exception:
                     continue
         finally:
@@ -495,6 +509,24 @@ class Database(DatabaseInterface):
     def has_channel(self):
         return True
 
+    def json_get(self, column, key=None):
+        column = Cast(column, 'jsonb')
+        if key:
+            column = JSONBExtractPath(column, key)
+        return column
+
+    def json_key_exists(self, column, key):
+        return JSONKeyExists(Cast(column, 'jsonb'), key)
+
+    def json_any_keys_exist(self, column, keys):
+        return JSONAnyKeyExist(Cast(column, 'jsonb'), keys)
+
+    def json_all_keys_exist(self, column, keys):
+        return JSONAllKeyExist(Cast(column, 'jsonb'), keys)
+
+    def json_contains(self, column, json):
+        return JSONContains(Cast(column, 'jsonb'), Cast(json, 'jsonb'))
+
 
 register_type(UNICODE)
 if PYDATE:
@@ -513,3 +545,8 @@ def convert_json(value):
     return json.loads(value, object_hook=JSONDecoder())
 register_default_json(loads=convert_json)
 register_default_jsonb(loads=convert_json)
+
+if is_gevent_monkey_patched():
+    from psycopg2.extensions import set_wait_callback
+    from psycopg2.extras import wait_select
+    set_wait_callback(wait_select)
