@@ -5,7 +5,10 @@ import logging
 import json
 
 from lxml import etree
+
+from trytond.i18n import gettext
 from trytond.model import ModelView, ModelSQL, fields
+from trytond.model.exceptions import ValidationError
 from trytond.pyson import Eval, Bool, PYSONDecoder, If
 from trytond.tools import file_open
 from trytond.transaction import Transaction
@@ -22,6 +25,10 @@ __all__ = [
 logger = logging.getLogger(__name__)
 
 
+class XMLError(ValidationError):
+    pass
+
+
 class View(ModelSQL, ModelView):
     "View"
     __name__ = 'ir.ui.view'
@@ -37,6 +44,7 @@ class View(ModelSQL, ModelView):
             ('graph', 'Graph'),
             ('calendar', 'Calendar'),
             ('board', 'Board'),
+            ('list-form', "List Form"),
             ], 'View Type', select=True,
         domain=[
             If(Bool(Eval('inherit')),
@@ -67,9 +75,6 @@ class View(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(View, cls).__setup__()
-        cls._error_messages.update({
-                'invalid_xml': 'Invalid XML for view "%s".',
-                })
         cls._order.insert(0, ('priority', 'ASC'))
         cls._buttons.update({
                 'show': {
@@ -96,6 +101,8 @@ class View(ModelSQL, ModelView):
         key = (cls.__name__, type_)
         rng = cls._get_rng_cache.get(key)
         if rng is None:
+            if type_ == 'list-form':
+                type_ = 'form'
             rng_name = os.path.join(os.path.dirname(__file__), type_ + '.rng')
             with open(rng_name, 'rb') as fp:
                 rng = etree.fromstring(fp.read())
@@ -131,8 +138,9 @@ class View(ModelSQL, ModelView):
                             validator.error_log.filter_from_errors()))
                     logger.error('Invalid XML view %s:\n%s\n%s',
                         view.rec_name, error_log, xml)
-                    cls.raise_user_error(
-                        'invalid_xml', (view.rec_name,), error_log)
+                    raise XMLError(
+                        gettext('ir.msg_view_invalid_xml', name=view.rec_name),
+                        error_log)
             root_element = tree.getroottree().getroot()
 
             # validate pyson attributes
@@ -154,8 +162,10 @@ class View(ModelSQL, ModelView):
                         logger.error(
                             'Invalid XML view %s:\n%s\n%s',
                             view.rec_name, error_log, xml)
-                        cls.raise_user_error(
-                            'invalid_xml', (view.rec_name,), error_log)
+                        raise XMLError(
+                            gettext(
+                                'ir.msg_view_invalid_xml', name=view.rec_name),
+                            error_log) from e
                 for child in element:
                     encode(child)
             encode(root_element)
@@ -201,7 +211,6 @@ class View(ModelSQL, ModelView):
 class ShowViewStart(ModelView):
     'Show view'
     __name__ = 'ir.ui.view.show.start'
-    __no_slots__ = True
 
 
 class ShowView(Wizard):
@@ -306,7 +315,7 @@ class ViewTreeState(ModelSQL, ModelView):
         super(ViewTreeState, cls).__setup__()
         cls.__rpc__.update({
                 'set': RPC(readonly=False, check_access=False),
-                'get': RPC(check_access=False),
+                'get': RPC(check_access=False, cache=dict(days=1)),
                 })
 
     @classmethod
