@@ -4,14 +4,14 @@
 import datetime
 import time
 import csv
-import warnings
 
 from decimal import Decimal
 from itertools import islice, chain
-from functools import reduce, wraps
+from functools import wraps
 from operator import itemgetter
 from collections import defaultdict
 
+from trytond.exceptions import UserError
 from trytond.model import Model
 from trytond.model import fields
 from trytond.tools import reduce_domain, memoize, is_instance_method, \
@@ -19,6 +19,7 @@ from trytond.tools import reduce_domain, memoize, is_instance_method, \
 from trytond.pyson import PYSONEncoder, PYSONDecoder, PYSON
 from trytond.const import OPERATORS
 from trytond.config import config
+from trytond.i18n import gettext
 from trytond.transaction import Transaction
 from trytond.pool import Pool
 from trytond.cache import LRUDict, LRUDictTransaction, freeze
@@ -29,6 +30,42 @@ from .descriptors import dualmethod
 __all__ = ['ModelStorage', 'EvalEnvironment']
 
 
+class AccessError(UserError):
+    pass
+
+
+class ImportDataError(UserError):
+    pass
+
+
+class ValidationError(UserError):
+    pass
+
+
+class DomainValidationError(ValidationError):
+    pass
+
+
+class RequiredValidationError(ValidationError):
+    pass
+
+
+class SizeValidationError(ValidationError):
+    pass
+
+
+class DigitsValidationError(ValidationError):
+    pass
+
+
+class SelectionValidationError(ValidationError):
+    pass
+
+
+class TimeFormatValidationError(ValidationError):
+    pass
+
+
 def without_check_access(func):
     @wraps(func)
     def wrapper(*args, **kwargs):
@@ -77,10 +114,10 @@ class ModelStorage(Model):
                     'search': RPC(result=lambda r: list(map(int, r))),
                     'search_count': RPC(),
                     'search_read': RPC(),
+                    'resources': RPC(instantiate=0, unique=False),
                     'export_data': RPC(instantiate=0, unique=False),
                     'import_data': RPC(readonly=False),
                     })
-        cls._constraints = []
 
     @staticmethod
     def default_create_uid():
@@ -128,23 +165,14 @@ class ModelStorage(Model):
                 Trigger.trigger_action(triggers, trigger)
 
     @classmethod
-    def read(cls, ids, fields_names=None):
+    def read(cls, ids, fields_names):
         '''
         Read fields_names of record ids.
-        If fields_names is None, it read all fields.
         The order is not guaranteed.
         '''
         pool = Pool()
         ModelAccess = pool.get('ir.model.access')
         ModelFieldAccess = pool.get('ir.model.field.access')
-
-        if not fields_names:
-            fields_names = []
-            for field_name in cls._fields.keys():
-                if ModelAccess.check_relation(cls.__name__, field_name,
-                        mode='read'):
-                    fields_names.append(field_name)
-
         ModelAccess.check(cls.__name__, 'read')
         ModelFieldAccess.check(cls.__name__, fields_names, 'read')
         return []
@@ -164,8 +192,9 @@ class ModelStorage(Model):
         all_fields = set()
         for records, values in zip(actions, actions):
             if not cls.check_xml_record(records, values):
-                cls.raise_user_error('write_xml_record',
-                        error_description='xml_record_desc')
+                raise AccessError(
+                    gettext('ir.msg_write_xml_record'),
+                    gettext('ir.msg_base_config_record'))
             all_records += records
             all_fields.update(values.keys())
 
@@ -232,12 +261,26 @@ class ModelStorage(Model):
         '''
         Delete records.
         '''
-        ModelAccess = Pool().get('ir.model.access')
+        pool = Pool()
+        ModelAccess = pool.get('ir.model.access')
+        ModelData = pool.get('ir.model.data')
 
         ModelAccess.check(cls.__name__, 'delete')
         if not cls.check_xml_record(records, None):
-            cls.raise_user_error('delete_xml_record',
-                    error_description='xml_record_desc')
+            raise AccessError(
+                gettext('ir.msg_delete_xml_record'),
+                gettext('ir.msg_base_config_record'))
+        if ModelData.has_model(cls.__name__):
+            with Transaction().set_context(_check_access=False):
+                data = []
+                for sub_records in grouped_slice(records):
+                    ids = [r.id for r in sub_records]
+                    data += ModelData.search([
+                            ('model', '=', cls.__name__),
+                            ('db_id', 'in', ids),
+                            ('noupdate', '=', True),
+                            ])
+                ModelData.write(data, {'db_id': None})
 
         # Increase transaction counter
         Transaction().counter += 1
@@ -347,18 +390,20 @@ class ModelStorage(Model):
                 or isinstance(f, fields.MultiValue))
             and n not in mptt]
         ids = list(map(int, records))
-        values = {d['id']: d for d in cls.read(ids, fields_names=fields_names)}
+        datas = cls.read(ids, fields_names=fields_names)
+        datas = dict((d['id'], d) for d in datas)
         field_defs = cls.fields_get(fields_names=fields_names)
         to_create = []
-        for id_ in ids:
-            data = convert_data(field_defs, values[id_])
+        for id in ids:
+            data = convert_data(field_defs, datas[id])
             to_create.append(data)
         new_records = cls.create(to_create)
+        id2new_record = dict(zip(ids, new_records))
 
         fields_translate = {}
         for field_name, field in field_defs.items():
-            if (field_name in cls._fields
-                    and getattr(cls._fields[field_name], 'translate', False)):
+            if field_name in cls._fields and \
+                    getattr(cls._fields[field_name], 'translate', False):
                 fields_translate[field_name] = field
 
         if fields_translate:
@@ -366,19 +411,16 @@ class ModelStorage(Model):
                 ('translatable', '=', True),
                 ])
             if langs:
-                id2new_records = defaultdict(list)
-                for id_, new_record in zip(ids, new_records):
-                    id2new_records[id_].append(new_record)
                 fields_names = list(fields_translate.keys()) + ['id']
                 for lang in langs:
                     # Prevent fuzzing translations when copying as the terms
                     # should be the same.
                     with Transaction().set_context(language=lang.code,
                             fuzzy_translation=False):
-                        values = cls.read(ids, fields_names=fields_names)
+                        datas = cls.read(ids, fields_names=fields_names)
                         to_write = []
-                        for data in values:
-                            to_write.append(id2new_records[data['id']])
+                        for data in datas:
+                            to_write.append([id2new_record[data['id']]])
                             to_write.append(
                                 convert_data(fields_translate, data))
                         cls.write(*to_write)
@@ -403,8 +445,10 @@ class ModelStorage(Model):
                 if relate:
                     if len(domain) >= 4:
                         target = pool.get(domain[3])
-                    else:
+                    elif hasattr(cls._fields[local], 'get_target'):
                         target = cls._fields[local].get_target()
+                    else:
+                        return
                     target_domain = [(relate,) + tuple(domain[1:])]
                     check_domain(target_domain, target, to_check)
             elif not domain:
@@ -420,7 +464,7 @@ class ModelStorage(Model):
             for oexpr, otype in order:
                 local, _, relate = oexpr.partition('.')
                 to_check[cls.__name__].add(local)
-                if relate:
+                if relate and hasattr(cls._fields[local], 'get_target'):
                     target = cls._fields[local].get_target()
                     target_order = [(relate, otype)]
                     check_order(target_order, target, to_check)
@@ -497,6 +541,24 @@ class ModelStorage(Model):
             return domain
         return process(domain)
 
+    def resources(self):
+        pool = Pool()
+        Attachment = pool.get('ir.attachment')
+        Note = pool.get('ir.note')
+
+        return {
+            'attachment_count': Attachment.search_count([
+                    ('resource', '=', str(self)),
+                    ]),
+            'note_count': Note.search_count([
+                    ('resource', '=', str(self)),
+                    ]),
+            'note_unread': Note.search_count([
+                    ('resource', '=', str(self)),
+                    ('unread', '=', True),
+                    ]),
+            }
+
     def get_rec_name(self, name):
         '''
         Return the rec_name of the instance.
@@ -639,10 +701,15 @@ class ModelStorage(Model):
                 ('rec_name', '=', value),
                 ], limit=2)
             if len(res) < 1:
-                cls.raise_user_error('relation_not_found', (value, relation))
+                raise ImportDataError(gettext(
+                        'ir.msg_relation_not_found',
+                        value=value,
+                        model=relation))
             elif len(res) > 1:
-                cls.raise_user_error('too_many_relations_found',
-                    (value, relation))
+                raise ImportDataError(
+                    gettext('ir.msg_too_many_relations_found',
+                        value=value,
+                        model=relation))
             else:
                 res = res[0].id
             return res
@@ -659,11 +726,15 @@ class ModelStorage(Model):
                     ('rec_name', '=', word),
                     ], limit=2)
                 if len(res2) < 1:
-                    cls.raise_user_error('relation_not_found',
-                        (word, relation))
+                    raise ImportDataError(
+                        gettext('ir.msg_relation_not_found',
+                            value=word,
+                            model=relation))
                 elif len(res2) > 1:
-                    cls.raise_user_error('too_many_relations_found',
-                        (word, relation))
+                    raise ImportDataError(
+                        gettext('ir.msg_too_many_relations_found',
+                            value=word,
+                            model=relation))
                 else:
                     res.extend(res2)
             if len(res):
@@ -680,17 +751,24 @@ class ModelStorage(Model):
             try:
                 relation, value = value.split(',', 1)
             except Exception:
-                cls.raise_user_error('reference_syntax_error',
-                    (value, '/'.join(field)))
+                raise ImportDataError(
+                    gettext('ir.msg_reference_syntax_error',
+                        value=value,
+                        field='/'.join(field)))
             Relation = pool.get(relation)
             res = Relation.search([
                 ('rec_name', '=', value),
                 ], limit=2)
             if len(res) < 1:
-                cls.raise_user_error('relation_not_found', (value, relation))
+                raise ImportDataError(gettext(
+                        'ir.msg_relation_not_found',
+                        value=value,
+                        model=relation))
             elif len(res) > 1:
-                cls.raise_user_error('too_many_relations_found',
-                    (value, relation))
+                raise ImportDataError(
+                    gettext('ir.msg_too_many_relations_found',
+                        value=value,
+                        model=relation))
             else:
                 res = '%s,%s' % (relation, res[0].id)
             return res
@@ -707,8 +785,10 @@ class ModelStorage(Model):
                 try:
                     relation, value = value.split(',', 1)
                 except Exception:
-                    cls.raise_user_error('reference_syntax_error',
-                        (value, '/'.join(field)))
+                    raise ImportDataError(
+                        gettext('ir.msg_reference_syntax_error',
+                            value=value,
+                            field='/'.join(field)))
                 value = [value]
             else:
                 value = [value]
@@ -717,8 +797,10 @@ class ModelStorage(Model):
                 try:
                     module, xml_id = word.rsplit('.', 1)
                 except Exception:
-                    cls.raise_user_error('xml_id_syntax_error',
-                        (word, '/'.join(field)))
+                    raise ImportDataError(
+                        gettext('ir.msg_xml_id_syntax_error',
+                            value=word,
+                            field='/'.join(field)))
                 db_id = ModelData.get_id(module, xml_id)
                 res_ids.append(db_id)
             if ftype == 'many2many' and res_ids:
@@ -727,7 +809,16 @@ class ModelStorage(Model):
                 return '%s,%s' % (relation, str(res_ids[0]))
             return res_ids and res_ids[0] or False
 
-        def process_lines(data, prefix, fields_def, position=0):
+        def dispatch(create, write, row, Relation=cls):
+            id_ = row.pop('id', None)
+            if id_:
+                write.append([Relation(id_)])
+                write.append(row)
+            else:
+                create.append(row)
+            return id_
+
+        def process_lines(data, prefix, fields_def, position=0, klass=cls):
             line = data[position]
             row = {}
             translate = {}
@@ -751,7 +842,12 @@ class ModelStorage(Model):
                     this_field_def = fields_def[field[-1]]
                     field_type = this_field_def['type']
                     res = None
-                    if field_type == 'boolean':
+                    if field[-1] == 'id':
+                        try:
+                            res = int(value)
+                        except ValueError:
+                            res = get_many2one(klass.__name__, value)
+                    elif field_type == 'boolean':
                         if value.lower() == 'true':
                             res = True
                         elif value.lower() == 'false':
@@ -761,19 +857,42 @@ class ModelStorage(Model):
                         else:
                             res = bool(int(value))
                     elif field_type == 'integer':
-                        res = int(value) if value else None
+                        if isinstance(value, int):
+                            res = value
+                        elif value:
+                            res = int(value)
+                        else:
+                            res = None
                     elif field_type == 'float':
-                        res = float(value) if value else None
+                        if isinstance(value, float):
+                            res = value
+                        elif value:
+                            res = float(value)
+                        else:
+                            res = None
                     elif field_type == 'numeric':
-                        res = Decimal(value) if value else None
+                        if isinstance(value, Decimal):
+                            res = value
+                        elif value:
+                            res = Decimal(value)
+                        else:
+                            res = None
                     elif field_type == 'date':
-                        res = (datetime.datetime.strptime(value,
-                                '%Y-%m-%d').date()
-                            if value else None)
+                        if isinstance(value, datetime.date):
+                            res = value
+                        elif value:
+                            res = datetime.datetime.strptime(
+                                value, '%Y-%m-%d').date()
+                        else:
+                            res = None
                     elif field_type == 'datetime':
-                        res = (datetime.datetime.strptime(value,
-                                '%Y-%m-%d %H:%M:%S')
-                            if value else None)
+                        if isinstance(value, datetime.datetime):
+                            res = value
+                        elif value:
+                            res = datetime.datetime.strptime(
+                                value, '%Y-%m-%d %H:%M:%S')
+                        else:
+                            res = None
                     elif field_type == 'many2one':
                         res = get_many2one(this_field_def['relation'], value)
                     elif field_type == 'many2many':
@@ -790,15 +909,13 @@ class ModelStorage(Model):
             # Import one2many fields
             nbrmax = 1
             for field in todo:
-                newfd = pool.get(fields_def[field]['relation']
-                        ).fields_get()
-                res = process_lines(data, prefix + [field], newfd,
-                        position)
-                (newrow, max2, _) = res
+                Relation = pool.get(fields_def[field]['relation'])
+                newfd = Relation.fields_get()
+                newrow, max2, _ = process_lines(
+                    data, prefix + [field], newfd, position, klass=Relation)
                 nbrmax = max(nbrmax, max2)
-                reduce(lambda x, y: x and y, newrow)
-                row[field] = (reduce(lambda x, y: x or y, list(newrow.values())) and
-                         [('create', [newrow])]) or []
+                create, write = [], []
+                dispatch(create, write, newrow, Relation)
                 i = max2
                 while (position + i) < len(data):
                     test = True
@@ -809,13 +926,17 @@ class ModelStorage(Model):
                             break
                     if not test:
                         break
-                    (newrow, max2, _) = \
-                        process_lines(data, prefix + [field], newfd,
-                            position + i)
-                    if reduce(lambda x, y: x or y, list(newrow.values())):
-                        row[field].append(('create', [newrow]))
+                    newrow, max2, _ = process_lines(
+                        data, prefix + [field], newfd, position + i,
+                        klass=Relation)
+                    dispatch(create, write, newrow, Relation)
                     i += max2
                     nbrmax = max(nbrmax, i)
+                row[field] = []
+                if create:
+                    row[field].append(('create', create))
+                if write:
+                    row[field].append(('write',) + tuple(write))
             if prefix_len == 0:
                 for i in range(max(nbrmax, 1)):
                     data.pop(0)
@@ -828,20 +949,35 @@ class ModelStorage(Model):
         fields_names = [x.split('/') for x in fields_names]
         fields_def = cls.fields_get()
 
-        to_create, translations, languages = [], [], set()
+        to_create, to_create_translations = [], []
+        to_write, to_write_translations = [], []
+        languages = set()
         while len(data):
-            (res, _, translate) = \
+            (row, _, translate) = \
                 process_lines(data, [], fields_def)
-            to_create.append(res)
-            translations.append(translate)
+            if dispatch(to_create, to_write, row):
+                to_write_translations.append(translate)
+            else:
+                to_create_translations.append(translate)
             languages.update(translate)
-        new_records = cls.create(to_create)
-        for language in languages:
-            translated = [t.get(language, {}) for t in translations]
-            with Transaction().set_context(language=language):
-                cls.write(*chain(*filter(itemgetter(1),
-                            zip(([r] for r in new_records), translated))))
-        return len(new_records)
+
+        def translate(records, translations):
+            for language in languages:
+                translated = [t.get(language, {}) for t in translations]
+                with Transaction().set_context(language=language):
+                    cls.write(*chain(*filter(itemgetter(1),
+                                zip(([r] for r in records), translated))))
+        count = 0
+        if to_create:
+            records = cls.create(to_create)
+            translate(records, to_create_translations)
+            count += len(records)
+        if to_write:
+            cls.write(*to_write)
+            records = sum(to_write[0:None:2], [])
+            translate(records, to_write_translations)
+            count += len(records)
+        return count
 
     @classmethod
     def check_xml_record(cls, records, values):
@@ -867,38 +1003,19 @@ class ModelStorage(Model):
                 ])
             if not models_data:
                 return True
-            if values is None:
-                return False
             for model_data in models_data:
-                if not model_data.values or model_data.noupdate:
-                    continue
-                xml_values = ModelData.load_values(model_data.values)
-                for key, val in values.items():
-                    if key in xml_values and val != xml_values[key]:
+                if values is None:
+                    if not model_data.noupdate:
                         return False
+                else:
+                    if not model_data.values or model_data.noupdate:
+                        continue
+                    xml_values = ModelData.load_values(model_data.values)
+                    for key, val in values.items():
+                        if key in xml_values and val != xml_values[key]:
+                            return False
         return True
 
-    @classmethod
-    def _get_error_args(cls, field_name):
-        pool = Pool()
-        ModelField = pool.get('ir.model.field')
-        error_args = {
-            'field': field_name,
-            'model': cls.__name__
-            }
-        if ModelField:
-            model_fields = ModelField.search([
-                        ('name', '=', field_name),
-                        ('model.model', '=', cls.__name__),
-                        ], limit=1)
-            if model_fields:
-                model_field, = model_fields
-                error_args.update({
-                        'field': model_field.field_description,
-                        'model': model_field.model.name,
-                        })
-        return error_args
-
     @classmethod
     def validate(cls, records):
         pass
@@ -911,19 +1028,6 @@ class ModelStorage(Model):
         with Transaction().set_context(_check_access=False):
             records = cls.browse(records)
 
-        def call(name):
-            method = getattr(cls, name)
-            if not is_instance_method(cls, name):
-                return method(records)
-            else:
-                return all(method(r) for r in records)
-        for field in cls._constraints:
-            warnings.warn(
-                '_constraints is deprecated, override validate instead',
-                DeprecationWarning, stacklevel=2)
-            if not call(field[0]):
-                cls.raise_user_error(field[1])
-
         ctx_pref = {}
         if Transaction().user:
             try:
@@ -1033,8 +1137,9 @@ class ModelStorage(Model):
                                 domain,
                                 ])
                     if sub_relations != set(finds):
-                        cls.raise_user_error('domain_validation_record',
-                            error_args=cls._get_error_args(field.name))
+                        raise DomainValidationError(
+                            gettext('ir.msg_domain_validation_record',
+                                **cls.__names__(field.name)))
 
         field_names = set(field_names or [])
         function_fields = {name for name, field in cls._fields.items()
@@ -1054,15 +1159,14 @@ class ModelStorage(Model):
                 validate_domain(field)
 
                 def required_test(value, field_name, field):
-                    if (isinstance(value, (type(None), type(False), list,
-                                    tuple, str, dict))
-                            and not value):
-                        cls.raise_user_error('required_validation_record',
-                            error_args=cls._get_error_args(field_name))
-                    if (field._type == 'reference'
-                            and not isinstance(value, ModelStorage)):
-                        cls.raise_user_error('required_validation_record',
-                            error_args=cls._get_error_args(field_name))
+                    if ((isinstance(value,
+                                    (type(None), bool, list, tuple, str, dict))
+                                and not value)
+                            or (field._type == 'reference'
+                                and not isinstance(value, ModelStorage))):
+                        raise RequiredValidationError(
+                            gettext('ir.msg_required_validation_record',
+                                **cls.__names__(field_name)))
                 # validate states required
                 if field.states and 'required' in field.states:
                     if is_pyson(field.states['required']):
@@ -1105,18 +1209,21 @@ class ModelStorage(Model):
                             field_size = field.size
                         size = len(getattr(record, field_name) or '')
                         if (size > field_size >= 0):
-                            error_args = cls._get_error_args(field_name)
+                            error_args = cls.__names__(field_name)
                             error_args['size'] = size
-                            cls.raise_user_error('size_validation_record',
-                                error_args=error_args)
+                            error_args['max_size'] = field_size
+                            raise SizeValidationError(
+                                gettext('ir.msg_size_validation_record',
+                                    **error_args))
 
                 def digits_test(value, digits, field_name):
-                    def raise_user_error(value):
-                        error_args = cls._get_error_args(field_name)
+                    def raise_error(value):
+                        error_args = cls.__names__(field_name)
                         error_args['digits'] = digits[1]
                         error_args['value'] = repr(value)
-                        cls.raise_user_error('digits_validation_record',
-                            error_args=error_args)
+                        raise DigitsValidationError(
+                            gettext('ir.msg_digits_validation_record',
+                                **error_args))
                     if (value is None
                             or not digits
                             or any(d is None for d in digits)):
@@ -1124,10 +1231,10 @@ class ModelStorage(Model):
                     if isinstance(value, Decimal):
                         exp = Decimal('.'.join(['0', '0' * digits[1]]))
                         if value.quantize(exp) != value:
-                            raise_user_error(value)
+                            raise_error(value)
                     else:
                         if not (round(value, digits[1]) == float(value)):
-                            raise_user_error(value)
+                            raise_error(value)
                 # validate digits
                 if getattr(field, 'digits', None):
                     if is_pyson(field.digits):
@@ -1170,10 +1277,11 @@ class ModelStorage(Model):
                             test.add('')
                             test.add(None)
                         if value not in test:
-                            error_args = cls._get_error_args(field_name)
+                            error_args = cls.__names__(field_name)
                             error_args['value'] = value
-                            cls.raise_user_error('selection_validation_record',
-                                error_args=error_args)
+                            raise SelectionValidationError(
+                                gettext('ir.msg_selection_validation_record',
+                                    **error_args))
 
                 def format_test(value, format, field_name):
                     if not value:
@@ -1182,10 +1290,11 @@ class ModelStorage(Model):
                         value = value.time()
                     if value != datetime.datetime.strptime(
                             value.strftime(format), format).time():
-                        error_args = cls._get_error_args(field_name)
+                        error_args = cls.__names__(field_name)
                         error_args['value'] = value
-                        cls.raise_user_error('time_format_validation_record',
-                            error_args=error_args)
+                        raise TimeFormatValidationError(
+                            gettext('ir.msg_time_format_validation_record',
+                                **error_args))
 
                 # validate time format
                 if (field._type in ('datetime', 'time')
@@ -1336,31 +1445,26 @@ class ModelStorage(Model):
             ifields = islice(ifields, 0, threshold)
             ffields.update(ifields)
 
-        require_context_field = False
         # add datetime_field
         for field in list(ffields.values()):
             if hasattr(field, 'datetime_field') and field.datetime_field:
-                require_context_field = True
-                if field.datetime_field not in ffields:
-                    datetime_field = self._fields[field.datetime_field]
-                    ffields[field.datetime_field] = datetime_field
+                datetime_field = self._fields[field.datetime_field]
+                ffields[field.datetime_field] = datetime_field
 
         # add depends of field with context
         for field in list(ffields.values()):
             if field.context:
                 eval_fields = fields.get_eval_fields(field.context)
                 for context_field_name in eval_fields:
-                    if context_field_name not in field.depends:
+                    if context_field_name in field.depends:
                         continue
-                    require_context_field = True
-                    if context_field_name not in ffields:
-                        context_field = self._fields.get(context_field_name)
+                    context_field = self._fields.get(context_field_name)
+                    if context_field not in ffields:
                         ffields[context_field_name] = context_field
 
         def filter_(id_):
-            return (id_ == self.id  # Ensure the value is read
-                or (name not in self._cache.get(id_, {})
-                    and name not in self._local_cache.get(id_, {})))
+            return (name not in self._cache.get(id_, {})
+                and name not in self._local_cache.get(id_, {}))
 
         def unique(ids):
             s = set()
@@ -1428,8 +1532,7 @@ class ModelStorage(Model):
                 self._transaction.set_user(self._user), \
                 self._transaction.reset_context(), \
                 self._transaction.set_context(self._context):
-            if (self.id in self._cache and name in self._cache[self.id]
-                    and not require_context_field):
+            if self.id in self._cache and name in self._cache[self.id]:
                 # Use values from cache
                 ids = islice(chain(islice(self._ids, index, None),
                         islice(self._ids, 0, max(index - 1, 0))),
@@ -1442,7 +1545,6 @@ class ModelStorage(Model):
                 read_data = self.read(list(ids), list(ffields.keys()))
             # create browse records for 'remote' models
             for data in read_data:
-                to_delete = set()
                 for fname, field in ffields.items():
                     fvalue = data[fname]
                     if field._type in ('many2one', 'one2one', 'one2many',
@@ -1461,11 +1563,10 @@ class ModelStorage(Model):
                             or field.context
                             or getattr(field, 'datetime_field', None)
                             or isinstance(field, fields.Function)):
-                        to_delete.add(fname)
+                        del data[fname]
                 if data['id'] not in self._cache:
                     self._cache[data['id']] = {}
-                self._cache[data['id']].update(
-                    **{k: v for k, v in data.items() if k not in to_delete})
+                self._cache[data['id']].update(data)
         return value
 
     @property
@@ -1475,6 +1576,8 @@ class ModelStorage(Model):
             return values
         for fname, value in self._values.items():
             field = self._fields[fname]
+            if isinstance(field, fields.Function) and not field.setter:
+                continue
             if field._type in ('many2one', 'one2one', 'reference'):
                 if value:
                     if ((value.id is None or value.id < 0)
