@@ -2,7 +2,6 @@
 # this repository contains the full copyright notices and license terms.
 import datetime
 import os
-import inspect
 import logging
 import subprocess
 import tempfile
@@ -230,6 +229,8 @@ class Report(URLMixin, PoolBase):
             def __str__(self):
                 return '%s,%s' % (Model.__name__, self.id)
 
+            def __unicode__(self):
+                return '%s,%s' % (Model.__name__, self.id)
         return [TranslateModel(id) for id in ids]
 
     @classmethod
@@ -295,27 +296,26 @@ class Report(URLMixin, PoolBase):
         return data
 
     @classmethod
-    def convert(cls, report, data):
+    def convert(cls, report, data, timeout=5 * 60):
         "converts the report data to another mimetype if necessary"
         input_format = report.template_extension
         output_format = report.extension or report.template_extension
 
-        if input_format == output_format and output_format in MIMETYPES:
+        if output_format in MIMETYPES:
             return output_format, data
 
         dtemp = tempfile.mkdtemp(prefix='trytond_')
         path = os.path.join(
             dtemp, report.report_name + os.extsep + input_format)
         oext = FORMAT2EXT.get(output_format, output_format)
-        mode = 'w+' if isinstance(data, str) else 'wb+'
-        with open(path, mode) as fp:
+        with open(path, 'wb+') as fp:
             fp.write(data)
         try:
             cmd = ['soffice',
                 '--headless', '--nolockcheck', '--nodefault', '--norestore',
                 '--convert-to', oext, '--outdir', dtemp, path]
             output = os.path.splitext(path)[0] + os.extsep + oext
-            subprocess.check_call(cmd)
+            subprocess.check_call(cmd, timeout=timeout)
             if os.path.exists(output):
                 with open(output, 'rb') as fp:
                     return oext, fp.read()
@@ -364,7 +364,7 @@ def get_email(report, record, languages):
     pool = Pool()
     ActionReport = pool.get('ir.action.report')
     report_id = None
-    if inspect.isclass(report) and issubclass(report, Report):
+    if isinstance(report, Report):
         Report_ = report
     else:
         if isinstance(report, ActionReport):
