@@ -10,10 +10,8 @@ import copy
 
 from trytond.pool import Pool, PoolBase
 from trytond.transaction import Transaction
-from trytond.error import WarningErrorMixin
 from trytond.url import URLMixin
 from trytond.protocols.jsonrpc import JSONDecoder, JSONEncoder
-from trytond.model import ModelSQL
 from trytond.model.fields import states_validate
 from trytond.pyson import PYSONEncoder
 from trytond.rpc import RPC
@@ -179,7 +177,7 @@ class StateReport(StateAction):
         return Action.get_action_values(action.type, [action.id])[0]
 
 
-class Wizard(WarningErrorMixin, URLMixin, PoolBase):
+class Wizard(URLMixin, PoolBase):
     start_state = 'start'
     end_state = 'end'
 
@@ -191,7 +189,6 @@ class Wizard(WarningErrorMixin, URLMixin, PoolBase):
             'delete': RPC(readonly=False),
             'execute': RPC(readonly=False, check_access=False),
             }
-        cls._error_messages = {}
 
         # Copy states
         for attr in dir(cls):
@@ -215,7 +212,6 @@ class Wizard(WarningErrorMixin, URLMixin, PoolBase):
         pool = Pool()
         Translation = pool.get('ir.translation')
         Translation.register_wizard(cls, module_name)
-        Translation.register_error_messages(cls, module_name)
 
     @classmethod
     def check_access(cls):
@@ -228,22 +224,18 @@ class Wizard(WarningErrorMixin, URLMixin, PoolBase):
         if Transaction().user == 0:
             return
 
-        with Transaction().set_context(_check_access=True):
-            model = context.get('active_model')
-            if model and model != 'ir.ui.menu':
-                ModelAccess.check(model, 'read')
-            groups = set(User.get_groups())
-            wizard_groups = ActionWizard.get_groups(cls.__name__,
-                action_id=context.get('action_id'))
-            if wizard_groups:
-                if not groups & wizard_groups:
-                    raise UserError('Calling wizard %s is not allowed!'
-                        % cls.__name__)
-            elif model and model != 'ir.ui.menu':
-                Model = pool.get(model)
-                if (not callable(getattr(Model, 'table_query', None))
-                        or Model.write.__func__ != ModelSQL.write.__func__):
-                    ModelAccess.check(model, 'write')
+        model = context.get('active_model')
+        if model:
+            ModelAccess.check(model, 'read')
+        groups = set(User.get_groups())
+        wizard_groups = ActionWizard.get_groups(cls.__name__,
+            action_id=context.get('action_id'))
+        if wizard_groups:
+            if not groups & wizard_groups:
+                raise UserError('Calling wizard %s is not allowed!'
+                    % cls.__name__)
+        elif model:
+            ModelAccess.check(model, 'write')
 
     @classmethod
     def create(cls):
